@@ -9,7 +9,7 @@
 import asyncio
 
 from clu.model import BaseModel, Property
-from clu.protocol import TCPStreamClient
+from clu.protocol import open_connection
 
 from .keys import KeysDictionary
 from .parser import ParseError, ReplyParser
@@ -21,14 +21,16 @@ __all__ = ['TronConnection', 'TronModel', 'TronKey']
 class TronKey(Property):
     """A Tron model key with callbacks.
 
-    Differs from `.Property` in that it is usually instantiated by
-    `.TronModel` with a ``key`` that is of type ``clu.legacy.keys.Key``
-    instead of a string.
+    Similar to `.Property` but stores the original key with the keyword
+    datamodel.
 
     """
 
-    def __repr__(self):
-        return f'<{self.__class__.__name__!s} ({self.key.name}): {self.value}>'
+    def __init__(self, name, value=[], key=None, model=None, callback=None):
+
+        super().__init__(name, value=value, model=model, callback=callback)
+
+        self.key = key
 
 
 class TronModel(BaseModel):
@@ -40,8 +42,9 @@ class TronModel(BaseModel):
         A dictionary of keys that define the datamodel.
     callback
         A function or coroutine to call when the datamodel changes. The
-        function is called with the instance of `TronModel` as the only
-        parameter. If the callback is a coroutine, it is scheduled as a task.
+        function is called with the instance of `.TronModel` and the
+        `.TronKey` that has changed. If the callback is a coroutine,
+        it is scheduled as a task.
     log : ~logging.Logger
         Where to log messages.
 
@@ -49,13 +52,13 @@ class TronModel(BaseModel):
 
     def __init__(self, keydict, callback=None, log=None):
 
-        super().__init__(keydict.name, callback, log=log)
+        super().__init__(keydict.name, callback=callback, log=log)
 
         self.keydict = keydict
 
         for key in self.keydict.keys:
             key = self.keydict.keys[key]
-            self[key.name] = TronKey(key, model=self)
+            self[key.name] = TronKey(key.name, key=key, model=self)
 
     def parse_reply(self, reply):
         """Parses a reply and updates the datamodel."""
@@ -79,10 +82,10 @@ class TronModel(BaseModel):
                                      f'{self.name}.{reply_key.name}.')
                 continue
 
-            self[reply_key.name].value = [value.native for value in reply_key.values]
+            self[key_name].value = [value.native for value in reply_key.values]
+            self[key_name].key = reply_key
 
-            if self.callback:
-                self.scheduler.add_callback(self.callback, self)
+            self.notify(self, self[key_name])
 
 
 class TronConnection(object):
@@ -90,13 +93,14 @@ class TronConnection(object):
 
     Parameters
     ----------
-    actor : str
-        The actor that is connecting to Tron. Used as the commander when
-        sending commands through the hub.
     host : str
         The host on which Tron is running.
     port : int
         The port on which Tron is running.
+    actor : str
+        The actor that is connecting to Tron. Used as the commander when
+        sending commands through the hub. If not specified the placeholder
+        ``client`` will be used.
     model_names : list
         A list of strings with the actors whose models will be tracked.
     log : ~logging.Logger
@@ -104,7 +108,7 @@ class TronConnection(object):
 
     """
 
-    def __init__(self, actor, host, port, model_names=None, log=None):
+    def __init__(self, host, port=6093, actor=None, model_names=None, log=None):
 
         self.actor = actor
 
@@ -126,8 +130,9 @@ class TronConnection(object):
                        for actor in model_names}
 
         self._parser = None
+        self.rparser = ReplyParser()
 
-        self.connection = None
+        self._client = None
 
     async def start(self, get_keys=True):
         """Starts the connection to Tron.
@@ -139,13 +144,21 @@ class TronConnection(object):
 
         """
 
-        self.connection = await TCPStreamClient(self.host, self.port)
+        self._client = await open_connection(self.host, self.port)
 
         self._parser = asyncio.create_task(self._parse_tron())
 
         if get_keys:
             asyncio.create_task(self.get_keys())
 
+        return self
+
+    def stop(self):
+        """Closes the connection."""
+
+        self._client.close()
+        self._parser.cancel()
+
     def send_command(self, target, command_string, mid=None):
         """Sends a command through the hub.
 
@@ -172,9 +185,12 @@ class TronConnection(object):
         # where commander needs to start with a letter and have a program and
         # a user joined by a dot. Otherwise the command will be accepted but
         # the reply will fail to parse.
-        command = f'{self.actor}.{self.actor} {mid} {target} {command_string}\n'
 
-        self.connection.writer.write(command.encode())
+        commander = self.actor.name if self.actor else 'client'
+
+        command = (f'{commander}.{commander} {mid} {target} {command_string}\n')
+
+        self._client.writer.write(command.encode())
 
         self._mid += 1
 
@@ -205,18 +221,15 @@ class TronConnection(object):
     async def _parse_tron(self):
         """Tracks new replies from Tron and updates the model."""
 
-        rparser = ReplyParser()
-
         while True:
 
-            line = await self.connection.reader.readline()
-
+            line = await self._client.reader.readline()
             try:
                 line = line.decode()  # Do not strip here or that will cause parsing problems.
-                reply = rparser.parse(line)
+                reply = self.rparser.parse(line)
             except ParseError:
                 if self.log:
-                    self.log.debug(f'failed parsing reply {line}.')
+                    self.log.debug(f'failed parsing reply {line.strip()}.')
                 continue
 
             actor = reply.header.actor
