@@ -8,6 +8,7 @@
 
 import asyncio
 import functools
+import inspect
 import re
 
 import click
@@ -20,8 +21,21 @@ __all__ = ['CluCommand', 'CluGroup', 'command_parser',
            'ClickParser', 'timeout']
 
 
+def coroutine(fn):
+    """Create a coroutine. Avoids deprecation of asyncio.coroutine in 3.10."""
+
+    if inspect.iscoroutinefunction(fn):
+        return fn
+
+    @functools.wraps(fn)
+    async def _wrapper(*args, **kwargs):
+        return fn(*args, **kwargs)
+
+    return _wrapper
+
+
 class CluCommand(click.Command):
-    """Override `click.Command` to pass the actor and command as arguments."""
+    """Override :py:class:`click.Command` to pass the actor and command."""
 
     def done_callback(self, task, exception_handler=None):
         """Checks if the command task has been successfully done."""
@@ -58,7 +72,7 @@ class CluCommand(click.Command):
         return True
 
     def invoke(self, ctx):
-        """Same as `click.Command.invoke` but passes the actor and command."""
+        """As :py:class:`click.Command.invoke` but passes the actor and command."""
 
         click.core._maybe_show_deprecated_notice(self)
 
@@ -70,7 +84,7 @@ class CluCommand(click.Command):
 
                 # Makes sure the callback is a coroutine
                 if not asyncio.iscoroutinefunction(self.callback):
-                    self.callback = asyncio.coroutine(self.callback)
+                    self.callback = coroutine(self.callback)
 
                 # Check to see if there is a timeout value in the callback.
                 # If so, schedules a task to be cancelled after timeout.
@@ -96,10 +110,14 @@ class CluCommand(click.Command):
 
 
 class CluGroup(click.Group):
-    """Override `click.Group` to make all child commands instances of `.CluCommand`."""
+    """Override :py:class:`click.Group`.
+
+    Makes all child commands instances of `.CluCommand`.
+
+    """
 
     def command(self, *args, **kwargs):
-        """Override `click.Group` to use `.CluCommand` as class by default."""
+        """Override :py:class:`click.Group` to use `.CluCommand` by default."""
 
         if 'cls' in kwargs:
             pass
